@@ -25,9 +25,9 @@ operations on the Accessible object, such as invoking default actions or
 moving the text caret.
 """
 
-__id__        = "$Id: braille.py 4034 2008-07-14 19:09:34Z wwalker $"
-__version__   = "$Revision: 4034 $"
-__date__      = "$Date: 2008-07-14 15:09:34 -0400 (Mon, 14 Jul 2008) $"
+__id__        = "$Id: braille.py 4171 2008-09-08 04:20:48Z joanied $"
+__version__   = "$Revision: 4171 $"
+__date__      = "$Date: 2008-09-08 00:20:48 -0400 (Mon, 08 Sep 2008) $"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -288,9 +288,9 @@ class Region:
         
         self.expandOnCursor = expandOnCursor
         
-        string = string.decode("UTF-8")
-        string = string.strip('\n')
-        self.rawLine = string.encode("UTF-8")
+        # The uncontracted string for the line.
+        #
+        self.rawLine = string.decode("UTF-8").strip("\n")
 
         if self.contracted:
             self.contractionTable = settings.brailleContractionTable or \
@@ -310,14 +310,25 @@ class Region:
         been scrolled off the display."""
         pass
 
-    def getAttributeMask(self):
+    def getAttributeMask(self, getLinkMask=True):
         """Creates a string which can be used as the attrOr field of brltty's
-        write structure for the purpose of indicating text attributes and
-        selection."""
+        write structure for the purpose of indicating text attributes, links,
+        and selection.
+
+        Arguments:
+        - getLinkMask: Whether or not we should take the time to get
+          the attributeMask for links. Reasons we might not want to
+          include knowning that we will fail and/or it taking an
+          unreasonable amount of time (AKA Gecko).
+        """
+
+        # Double check for ellipses.
+        #
+        maskSize = len(self.string) + (2 * self.string.count(u'\u2026'))
 
         # Create an empty mask.
         #
-        mask = ['\x00'] * len(self.string)
+        mask = ['\x00'] * maskSize
         return "".join(mask)
     
     def repositionCursor(self):
@@ -347,12 +358,12 @@ class Region:
         if not expandOnCursor or cursorOnSpace:
             contracted, inPos, outPos, cursorPos = \
                              louis.translate([self.contractionTable],
-                                             line.decode(),
+                                             line,
                                              cursorPos=cursorOffset)
         else:
             contracted, inPos, outPos, cursorPos = \
                              louis.translate([self.contractionTable],
-                                             line.decode(),
+                                             line,
                                              cursorPos=cursorOffset,
                                              mode=louis.MODE.compbrlAtCursor)
 
@@ -469,6 +480,10 @@ class Text(Region):
             string = ""
 
         string = string.decode("UTF-8")
+        if label:
+            label = label.decode("UTF-8")
+        if eol:
+            eol = eol.decode("UTF-8")
 
         try:
             endOffset = endOffset - self.lineOffset
@@ -486,7 +501,7 @@ class Text(Region):
 
         cursorOffset = min(self.caretOffset - self.lineOffset, len(string))
 
-        self._maxCaretOffset = self.lineOffset + len(string.decode("UTF-8"))
+        self._maxCaretOffset = self.lineOffset + len(string)
 
         self.eol = eol
 
@@ -513,6 +528,8 @@ class Text(Region):
 
         [string, caretOffset, lineOffset] = \
                  orca_state.activeScript.getTextLineAtCaret(self.accessible)
+        string = string.decode("UTF-8")
+
         cursorOffset = min(caretOffset - lineOffset, len(string))
         
         if lineOffset != self.lineOffset:
@@ -546,10 +563,17 @@ class Text(Region):
         orca_state.activeScript.setCaretOffset(
             self.accessible, newCaretOffset)
 
-    def getAttributeMask(self):
+    def getAttributeMask(self, getLinkMask=True):
         """Creates a string which can be used as the attrOr field of brltty's
-        write structure for the purpose of indicating text attributes and
-        selection."""
+        write structure for the purpose of indicating text attributes, links,
+        and selection.
+
+        Arguments:
+        - getLinkMask: Whether or not we should take the time to get
+          the attributeMask for links. Reasons we might not want to
+          include knowning that we will fail and/or it taking an
+          unreasonable amount of time (AKA Gecko).
+        """
 
         try:
             text = self.accessible.queryText()
@@ -567,7 +591,7 @@ class Text(Region):
         linkIndicator = settings.brailleLinkIndicator
         script = orca_state.activeScript
 
-        if linkIndicator != settings.BRAILLE_LINK_NONE:
+        if getLinkMask and linkIndicator != settings.BRAILLE_LINK_NONE:
             try:
                 hyperText = self.accessible.queryHypertext()
                 nLinks = hyperText.getNLinks()
@@ -723,12 +747,18 @@ class Line:
     def addRegions(self, regions):
         self.regions.extend(regions)
 
-    def getLineInfo(self):
+    def getLineInfo(self, getLinkMask=True):
         """Computes the complete string for this line as well as a
         0-based index where the focused region starts on this line.
         If the region with focus is not on this line, then the index
         will be -1.
 
+        Arguments:
+        - getLinkMask: Whether or not we should take the time to get
+          the attributeMask for links. Reasons we might not want to
+          include knowning that we will fail and/or it taking an
+          unreasonable amount of time (AKA Gecko).
+
         Returns [string, offsetIndex, attributeMask]
         """
 
@@ -737,14 +767,14 @@ class Line:
         attributeMask = ""
         for region in self.regions:
             if region == _regionWithFocus:
-                focusOffset = len(string.decode("UTF-8"))
+                focusOffset = len(string)
             if region.string:
-                # [[[TODO: WDW - HACK: Replace UTF-8 ellipses with "..."
+                # [[[TODO: WDW - HACK: Replace ellipses with "..."
                 # The ultimate solution is to get i18n support into
                 # BrlTTY.]]]
                 #
-                string += region.string.replace("\342\200\246", "...")
-            mask = region.getAttributeMask()
+                string += region.string.replace(u'\u2026', "...")
+            mask = region.getAttributeMask(getLinkMask)
             attributeMask += mask
 
         return [string, focusOffset, attributeMask]
@@ -767,12 +797,12 @@ class Line:
         for region in self.regions:
             foundRegion = region
             string = string + region.string
-            if len(string.decode("UTF-8")) > offset:
+            if len(string) > offset:
                 break
             else:
-                pos = len(string.decode("UTF-8"))
+                pos = len(string)
 
-        if offset >= len(string.decode("UTF-8")):
+        if offset >= len(string):
             return [None, -1]
         else:
             return [foundRegion, offset - pos]
@@ -840,13 +870,19 @@ def getShowingLine():
     """
     return _lines[viewport[1]]
 
-def setFocus(region, panToFocus=True):
+def setFocus(region, panToFocus=True, getLinkMask=True):
     """Specififes the region with focus.  This region will be positioned
     at the home position if panToFocus is True.
 
     Arguments:
     - region: the given region, which much be in a line that has been
-              added to the logical display
+      added to the logical display
+    - panToFocus: whether or not to position the region at the home
+      position
+    - getLinkMask: Whether or not we should take the time to get the
+      attributeMask for links. Reasons we might not want to include
+      knowning that we will fail and/or it taking an unreasonable
+      amount of time (AKA Gecko).
     """
 
     global _regionWithFocus
@@ -877,7 +913,7 @@ def setFocus(region, panToFocus=True):
             lineNum += 1
 
     line = _lines[viewport[1]]
-    [string, offset, attributeMask] = line.getLineInfo()
+    [string, offset, attributeMask] = line.getLineInfo(getLinkMask)
 
     # If the cursor is too far right, we scroll the viewport
     # so the cursor will be on the last cell of the display.
@@ -887,7 +923,7 @@ def setFocus(region, panToFocus=True):
 
     viewport[0] = max(0, offset)
 
-def refresh(panToCursor=True, targetCursorCell=0):
+def refresh(panToCursor=True, targetCursorCell=0, getLinkMask=True):
     """Repaints the Braille on the physical display.  This clips the entire
     logical structure by the viewport and also sets the cursor to the
     appropriate location.  [[[TODO: WDW - I'm not sure how BrlTTY handles
@@ -897,14 +933,17 @@ def refresh(panToCursor=True, targetCursorCell=0):
     Arguments:
 
     - panToCursor: if True, will adjust the viewport so the cursor is
-                   showing.
+      showing.
     - targetCursorCell: Only effective if panToCursor is True.
-                        0 means automatically place the cursor somewhere
-                        on the display so as to minimize movement but
-                        show as much of the line as possible.  A positive
-                        value is a 1-based target cell from the left side
-                        of the display and a negative value is a 1-based
-                        target cell from the right side of the display.
+      0 means automatically place the cursor somewhere on the display so
+      as to minimize movement but show as much of the line as possible.
+      A positive value is a 1-based target cell from the left side of
+      the display and a negative value is a 1-based target cell from the
+      right side of the display.
+    - getLinkMask: Whether or not we should take the time to get the
+      attributeMask for links. Reasons we might not want to include
+      knowning that we will fail and/or it taking an unreasonable
+      amount of time (AKA Gecko).
     """
 
     global endIsShowing
@@ -932,7 +971,7 @@ def refresh(panToCursor=True, targetCursorCell=0):
     # actually is in the string.
     #
     line = _lines[viewport[1]]
-    [string, focusOffset, attributeMask] = line.getLineInfo()
+    [string, focusOffset, attributeMask] = line.getLineInfo(getLinkMask)
     cursorOffset = -1
     if focusOffset >= 0:
         cursorOffset = focusOffset + _regionWithFocus.cursorOffset
@@ -943,7 +982,7 @@ def refresh(panToCursor=True, targetCursorCell=0):
     # right of the display if we need to pan right.
     #
     if panToCursor and (cursorOffset >= 0):
-        if len(string.decode("UTF-8")) <= _displaySize[0]:
+        if len(string) <= _displaySize[0]:
             viewport[0] = 0
         elif targetCursorCell:
             viewport[0] = max(0, cursorOffset - targetCursorCell + 1)
@@ -966,19 +1005,18 @@ def refresh(panToCursor=True, targetCursorCell=0):
 
     logLine = "BRAILLE LINE:  '%s'" % string
     debug.println(debug.LEVEL_INFO, logLine)
-    log.info(logLine)
+    log.info(logLine.encode("UTF-8"))
     logLine = "     VISIBLE:  '%s', cursor=%d" % \
                     (string[startPos:endPos], cursorCell)
     debug.println(debug.LEVEL_INFO, logLine)
-    log.info(logLine)
+    log.info(logLine.encode("UTF-8"))
 
-    string = string.decode("UTF-8")
-    substring = string[startPos:endPos].encode("UTF-8")
+    substring = string[startPos:endPos]
     if useBrlAPIBindings:
         if brlAPIRunning:
             writeStruct = brlapi.WriteStruct()
             writeStruct.regionBegin = 1
-            writeStruct.regionSize = len(substring.decode("UTF-8"))
+            writeStruct.regionSize = len(substring)
             while writeStruct.regionSize < _displaySize[0]:
                 substring += " "
                 if attributeMask:
@@ -986,7 +1024,6 @@ def refresh(panToCursor=True, targetCursorCell=0):
                 writeStruct.regionSize += 1
             writeStruct.text = substring
             writeStruct.cursor = cursorCell
-            writeStruct.charset = "UTF-8"
 
             # [[[WDW - if you want to muck around with the dots on the
             # display to do things such as add underlines, you can use
@@ -1104,7 +1141,7 @@ def panRight(panAmount=0):
         lineNum = viewport[1]
         newX = viewport[0] + panAmount
         [string, focusOffset, attributeMask] = _lines[lineNum].getLineInfo()
-        if newX < len(string.decode("UTF-8")):
+        if newX < len(string):
             viewport[0] = newX
 
     return oldX != viewport[0]
