@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2005-2006 Google Inc.
+# Copyright 2005-2008 Google Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -14,8 +14,8 @@
 #
 # You should have received a copy of the GNU Library General Public
 # License along with this library; if not, write to the
-# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-# Boston, MA 02111-1307, USA.
+# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
+# Boston MA  02110-1301 USA.
 #
 """Python wrapper for Emacspeak speech servers.
 
@@ -38,11 +38,11 @@ emacspeak modules:
 
 """
 
-__id__ = "$Id: espeechfactory.py 3082 2007-10-17 21:37:36Z richb $"
+__id__ = "$Id: espeechfactory.py 4180 2008-09-08 22:00:14Z wwalker $"
 __author__ = "T. V. Raman"
-__version__ = "$Revision: 3082 $"
-__date__ = "$Date: 2007-10-17 17:37:36 -0400 (Wed, 17 Oct 2007) $"
-__copyright__ = "Copyright (c) 2005 Google Inc."
+__version__ = "$Revision: 4180 $"
+__date__ = "$Date: 2008-09-08 18:00:14 -0400 (Mon, 08 Sep 2008) $"
+__copyright__ = "Copyright (c) 2005-2008 Google Inc."
 __license__ = "LGPL"
 __all__ = ['Speaker']
 
@@ -115,9 +115,10 @@ class SpeechServer(speechserver.SpeechServer):
 
         f = open(os.path.join(SpeechServer.location, '.servers'))
         for line in f:
-            if line[0] == '#' or line.strip() == '': continue
+            if line[0] == '#' or line.strip() == '':
+                continue
             name = line.strip()
-            if not SpeechServer.__activeServers.has_key(name):
+            if name not in SpeechServer.__activeServers:
                 try:
                     SpeechServer.__activeServers[name] = SpeechServer(name)
                 except:
@@ -128,11 +129,11 @@ class SpeechServer(speechserver.SpeechServer):
 
     getSpeechServers = staticmethod(getSpeechServers)
 
-    def getSpeechServer(info=['outloud','outloud']):
+    def getSpeechServer(info=['outloud', 'outloud']):
         """Gets a given SpeechServer based upon the info.
         See SpeechServer.getInfo() for more info.
         """
-        if SpeechServer.__activeServers.has_key(info[0]):
+        if info[0] in SpeechServer.__activeServers:
             return SpeechServer.__activeServers[info[0]]
         else:
             try:
@@ -149,6 +150,7 @@ class SpeechServer(speechserver.SpeechServer):
         for key in SpeechServer.__activeServers.keys():
             server = SpeechServer.__activeServers[key]
             server.shutdown()
+        SpeechServer.__getSpeechServersCalled = False
 
     shutdownActiveServers = staticmethod(shutdownActiveServers)
 
@@ -182,13 +184,14 @@ class SpeechServer(speechserver.SpeechServer):
             self._settings.update(initial)
             self.configure(self._settings)
 
-    def configure(self, settings):
+    def configure(self, engineSettings):
         """Configure engine with settings."""
-        for k in settings.keys():
-            if hasattr(self, k) and callable(getattr(self,k)):
-                getattr(self,k)(settings[k])
+        for k in engineSettings.keys():
+            if hasattr(self, k) and callable(getattr(self, k)):
+                getattr(self, k)(engineSettings[k])
 
-    def settings(self): return self._settings
+    def settings(self):
+        return self._settings
 
     def getInfo(self):
         """Returns [driverName, serverId]
@@ -209,7 +212,6 @@ class SpeechServer(speechserver.SpeechServer):
                 families.append(speechserver.VoiceFamily(props))
         except:
             debug.printException(debug.LEVEL_SEVERE)
-            pass
 
         return families
 
@@ -235,15 +237,15 @@ class SpeechServer(speechserver.SpeechServer):
         """Speak single character."""
         self._output.write("l {%s}\n" % character)
 
-    def speakUtterances(self, list, acss=None, interrupt=True):
+    def speakUtterances(self, utteranceList, acss=None, interrupt=True):
         """Speak list of utterances."""
         if acss:
             code = self.getvoice(acss)
-            for t in list:
+            for t in utteranceList:
                 self._output.write("q { %s %s %s }\n" % \
                                    (code[0], str(t), code[1]))
         else:
-            for t in list:
+            for t in utteranceList:
                 self._output.write("q { %s }\n" % str(t))
         self._output.write("d\n")
 
@@ -287,7 +289,7 @@ class SpeechServer(speechserver.SpeechServer):
 
     def shutdown(self):
         """Shutdown speech engine."""
-        if SpeechServer.__activeServers.has_key(self._engine):
+        if self._engine in SpeechServer.__activeServers:
             self._output.close()
             del SpeechServer.__activeServers[self._engine]
 
@@ -355,11 +357,11 @@ def _test():
     s = SpeechServer()
     a = acss.ACSS()
     s.punctuations('some')
-    s.queueText("This is an initial test.");
+    s.queueText("This is an initial test.")
     s.queueText("Next, we'll test audio formatted output.")
     for d in ['average-pitch', 'pitch-range',
               'richness', 'stress']:
-        for i in range(0,10,2):
+        for i in range(0, 10, 2):
             a[d] = i
             s.queueText("Set %s to %i. " % (d, i), a)
         del a[d]
@@ -370,4 +372,5 @@ def _test():
     s.shutdown()
 
 
-if __name__ == "__main__": _test()
+if __name__ == "__main__":
+    _test()
