@@ -20,9 +20,9 @@
 """ Custom script for Thunderbird 3.
 """
 
-__id__        = "$Id: $"
-__version__   = "$Revision: $"
-__date__      = "$Date: $"
+__id__        = "$Id: script.py 4221 2008-09-15 08:11:23Z wwalker $"
+__version__   = "$Revision: 4221 $"
+__date__      = "$Date: 2008-09-15 04:11:23 -0400 (Mon, 15 Sep 2008) $"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -156,6 +156,55 @@ class Script(Gecko.Script):
 
         return False
 
+    def onCaretMoved(self, event):
+        """Called whenever the caret moves.
+
+        Arguments:
+        - event: the Event
+        """
+
+        # Page_Up/Page_Down are not used by Orca. However, users report
+        # using these keys in Thunderbird without success. The default
+        # script is sometimes rejecting the resulting caret-moved events
+        # based on the locusOfFocus other times Gecko is because of the
+        # caret context.
+        #
+        updatePosition = False
+        if isinstance(orca_state.lastInputEvent, input_event.KeyboardEvent):
+            string = orca_state.lastNonModifierKeyEvent.event_string
+            updatePosition = string in ["Page_Up", "Page_Down"]
+
+        # Unlike the unpredictable wild, wild web, odds are good that a
+        # caret-moved event in a message composition window is valid. But
+        # depending upon the locusOfFocus at the time this event is issued
+        # the default Gecko toolkit script might not do the right thing.
+        #
+        if not updatePosition and event.detail1 >= 0:
+            updatePosition = \
+                event.source.getState().contains(pyatspi.STATE_EDITABLE)
+
+        if updatePosition:
+            orca.setLocusOfFocus(event, event.source, False)
+            self.setCaretContext(event.source, event.detail1)
+
+            # The Gecko script, should it be about to pass along this
+            # event to the default script, will set the locusOfFocus to
+            # the object returned by findFirstCaretContext(). If that
+            # object is not the same as the event source or the event
+            # source's parent, the default script will reject the event.
+            # As a result, if the user presses Page_Up or Page_Down and
+            # just so happens to land on an object whose sole contents
+            # is an image, we'll say nothing. Ultimately this should
+            # probably be handled elsewhere, but this close to the next
+            # major (2.24) release, I (JD) am not risking it. :-)
+            #
+            [obj, offset] = \
+                self.findFirstCaretContext(event.source, event.detail1)
+            if obj.getRole() == pyatspi.ROLE_IMAGE:
+                return default.Script.onCaretMoved(self, event)
+
+        return Gecko.Script.onCaretMoved(self, event)
+
     def onFocus(self, event):
         """ Called whenever an object gets focus.
 
@@ -250,10 +299,31 @@ class Script(Gecko.Script):
         # Handle a newly-opened message.
         #
         if event.source.getRole() == pyatspi.ROLE_DOCUMENT_FRAME \
-           and orca_state.locusOfFocus.getRole() == pyatspi.ROLE_FRAME \
-           and self._messageLoaded:
-            consume = True
-            self._presentMessage(event.source)
+           and orca_state.locusOfFocus.getRole() == pyatspi.ROLE_FRAME:
+            if self._messageLoaded:
+                consume = True
+                self._presentMessage(event.source)
+
+            # If the user just gave focus to the message window (e.g. by
+            # Alt+Tabbing back into it), we might have an existing caret
+            # context. But we'll need the document frame in order to verify
+            # this. Therefore try to find the document frame.
+            #
+            elif self.getCaretContext() == [None, -1]:
+                documentFrame = None
+                for child in orca_state.locusOfFocus:
+                    if child.getRole() == pyatspi.ROLE_INTERNAL_FRAME \
+                       and child.childCount \
+                       and child[0].getRole() == pyatspi.ROLE_DOCUMENT_FRAME:
+                        documentFrame = child[0]
+                        break
+                try:
+                    contextObj, contextOffset = \
+                        self._documentFrameCaretContext[hash(documentFrame)]
+                    if contextObj:
+                        orca.setLocusOfFocus(event, contextObj)
+                except:
+                    pass
 
         if not consume:
             Gecko.Script.onFocus(self, event)
